<?php

/**
 * @file
 * Contains \Drupal\blocache\BlocacheMetadata.
 */

namespace Drupal\blocache;

use Drupal\block\BlockInterface;

/**
 * Class BlocacheMetadata.
 */
class BlocacheMetadata {

	/**
   * 
   *
   * @var string
   */
	const OVERRIDEN = 'overridden';

	/**
   * 
   *
   * @var string
   */
	const MODULE = 'blocache'; 

	/**
   * 
   *
   * @var string
   */
	const METADATA_MAX_AGE = 'max-age';

	/**
   * 
   *
   * @var string
   */
	const METADATA_CONTEXTS = 'contexts';

	/**
   * 
   *
   * @var string
   */
	const METADATA_TAGS = 'tags';

	/**
   * 
   *
   * @var \Drupal\block\BlockInterface
   */
	protected $block = NULL;

	/**
   * 
   *
   * @var boolean
   */
	protected $overridden = FALSE;
	
	/**
   * 
   *
   * @var array
   */
	protected $defaults = [];

	/**
   * 
   *
   * @var array
   */
	protected $overrides = [];

	/**
	 * 
	 *
	 * @return 
	 * 
	 */
	public function getMetadata() {
		if ($this->isOverridden()) {
			return $this->getOverrides();
		}

		return $this->getDefaults();
	}

	/**
	 *
	 *
	 * @return 
	 * 
	 */
	public function getDefaults() {
		return $this->defaults;
	}

	/**
	 *
	 *
	 * @return 
	 * 
	 */
	public function getOverrides() {
		return $this->overrides;
	}

	/**
	 *
	 * @param $block
	 * 
	 */
	public function setBlock(BlockInterface $block) {
		$this->block = $block;
		
		$this->defaults = [
      self::METADATA_MAX_AGE => $this->block->getCacheMaxAge(),
      self::METADATA_CONTEXTS => $this->block->getCacheContexts(),
      self::METADATA_TAGS => $this->block->getCacheTags(),
    ];

		$this->overridden = $this->block->getThirdPartySetting(self::MODULE, self::OVERRIDEN);
		$this->overrides = $this->block->getThirdPartySetting(self::MODULE, 'metadata');
	}

	/**
	 *
	 *
	 * @return 
	 * 
	 */
	public function setOverrides(int $max_age, array $contexts, array $tags) {
		if (!$this->isBlock()) {
			return FALSE;
		}

		$metadata = [
			self::METADATA_MAX_AGE => $max_age,
 			self::METADATA_CONTEXTS => array_filter($contexts),
 			self::METADATA_TAGS => array_filter($tags),
		];

		$this->overridden = TRUE;
		$this->overrides = $metadata;
    $this->block->setThirdPartySetting(self::MODULE, self::OVERRIDEN, TRUE);
  	$this->block->setThirdPartySetting(self::MODULE, 'metadata', $metadata);

    return TRUE;
	}

	/**
	 *
	 *
	 * @return 
	 * 
	 */
	public function unsetOverrides() {
		if (!$this->isBlock()) {
			return FALSE;
		}

		$this->overridden = FALSE;    
  	$this->overrides = [];
    $this->block->unsetThirdPartySetting(self::MODULE, self::OVERRIDEN);
    $this->block->unsetThirdPartySetting(self::MODULE, 'metadata');

    return TRUE;
	}

	/**
	 *
	 *
	 * @return 
	 * 
	 */
	public function isOverridden() {
		if (!$this->isBlock()) {
			return FALSE;
		}

		return $this->overridden;		
	}


	/**
	 *
	 *
	 * @return 
	 * 
	 */
	private function isBlock() {
		return $this->block instanceof \Drupal\block\BlockInterface;
	}


}